/*-------------------------------------------------------------------

    BRUTE.C

    20.4.2001

    Ismo Krkkinen

    Brute force search using RLS.

-------------------------------------------------------------------*/

#include "brute.h"
#include "cb_util.h"
#include "rls.h"
#include "cb.h"
#include "interfc.h"
#include <string.h>

static void PrintOutput(int Level, int M, int Iter, float Best, float Current) {
    switch (Level) {
    case 0: break;
    case 1:
	if (Best == Current) PrintMessage("M = %i, best = %#.6g.\n", M, Best);
	break;
    case 2:
	PrintMessage("M = %i, best = %#9.6g, current = %#9.6g, iter = %i.\n",
	    M, Best, Current, Iter);
	break;
    case 3:
	PrintMessage("M = %i, best = %#9.6g, current = %#9.6g, iter = %i.\n",
	    M, Best, Current, Iter);
	break;
    default:
	PrintMessage("M = %i, best = %#9.6g.\n", M, Best);
	break;
    }
}

int BruteForce(TRAININGSET* TSIn, CODEBOOK* CBOut, PARTITIONING* PAOut,
    int MinGroups, int MaxGroups, CriterionInfo* CI, float* Error,
    Iterations* Iter, int GLAIterations,
    int SwapMethod, int Output, int Trace, int TraceID)
{
    int Interrupt, k, RLS_Output;
    float Curr;
    CODEBOOK CurrCB;
    PARTITIONING CurrPA;
    if (Output > 2) RLS_Output = Output - 2;
    else RLS_Output = 0;
    SetExitRequestHonored(1);
    CreateNewCodebook(CBOut, MinGroups, TSIn);
    AddGenerationMethod(CBOut, "Brute force");
    CreateNewPartitioning(PAOut, TSIn, MinGroups);
    GenerateRandomCodebook(TSIn, CBOut);
    VectorFreq(CBOut, 0) = TotalFreq(TSIn);
    GenerateOptimalPartitioning(TSIn, CBOut, PAOut);
    GenerateOptimalCodebook(TSIn, CBOut, PAOut);
    Interrupt = RandomizedLocalSearch(TSIn, CBOut, PAOut, CI, Error,
	Iter, GLAIterations, SwapMethod, RLS_Output, Trace, TraceID);
    PrintOutput(Output, MinGroups, Iter->Count, *Error, *Error);
    if (Interrupt) return Interrupt;
    CreateNewCodebook(&CurrCB, MaxGroups, TSIn);
    AddGenerationMethod(&CurrCB, "Brute force");
    CreateNewPartitioning(&CurrPA, TSIn, MaxGroups);
    DecreaseCodebookSize(&CurrCB, MinGroups);
    DecreaseNumberOfPartitions(&CurrPA, MinGroups);
    for (k = MinGroups + 1; k <= MaxGroups; k++) {
	IncreaseCodebookSize(&CurrCB, k);
	IncreaseNumberOfPartitions(&CurrPA, k);
        GenerateRandomCodebook(TSIn, &CurrCB);
	GenerateOptimalPartitioning(TSIn, &CurrCB, &CurrPA);
	GenerateOptimalCodebook(TSIn, &CurrCB, &CurrPA);
        itReset(Iter);
	Interrupt = RandomizedLocalSearch(TSIn, &CurrCB, &CurrPA,
	    CI, &Curr, Iter, GLAIterations, SwapMethod,
	    RLS_Output, Trace, TraceID);
	if (Interrupt) {
	    FreeCodebook(&CurrCB);
	    FreePartitioning(&CurrPA);
	    return Interrupt;
	}
        if (*Error > Curr) {
	    IncreaseCodebookSize(CBOut, BookSize(&CurrCB));
            CopyCodebook(&CurrCB, CBOut);
	    IncreaseNumberOfPartitions(PAOut, BookSize(&CurrCB));
            CopyPartitioning(&CurrPA, PAOut);
            *Error = Curr;
        }
	PrintOutput(Output, k, Iter->Count, *Error, Curr);
    }
    FreeCodebook(&CurrCB);
    FreePartitioning(&CurrPA);
    return 0;
}

